searchData={"items":[{"type":"module","title":"SMPPEX.ESME","doc":"This is a module for launching an `SMPPEX.Session` implementation as an ESME.\n\nTo start an ESME one generally should do the following.\n\n1. Implement an `SMPPEX.Session` behaviour.\n\n```elixir\n\ndefmodule MyESMESession do\n  use SMPPEX.Session\n\n  # ...Callback implementation\n\nend\n\n```\n\n2. Launch it.\n\n```elixir\n\n{:ok, esme_session} = SMPPEX.ESME.start_link(\"127.0.0.1\",\n                                             2775,\n                                             {MyESMESession, some_args})\n\n```","ref":"SMPPEX.ESME.html"},{"type":"function","title":"SMPPEX.ESME.start_link/4","doc":"Starts an `SMPPEX.Session` implementation as an ESME entitiy,\ni.e. makes a transport connection to `host`:`port` and starts an `SMPPEX.Session`\nto handle the connection with the passed module.\n\nThe function does not return until ESME successfully connects to the specified\n`host` and `port` and initializes or fails.\n`module` is the callback module which should implement `SMPPEX.ESME` behaviour.\n`args` is the argument passed to the `init` callback.\n`opts` is a keyword list of different options:\n* `:transport` is Ranch transport used for TCP connection: either `:ranch_tcp` (the default) or\n`:ranch_ssl`;\n* `:session_module` is a module to use as an alternative to `SMPPEX.Session`\nfor handling sessions (if needed). For example, `SMPPEX.TelemetrySession`.\n* `:timeout` is timeout for transport connect. The default is 5000 ms;\n* `:esme_opts` is a keyword list of ESME options:\n    - `:enquire_link_limit` is value for enquire_link SMPP timer, i.e. the interval of SMPP session inactivity after which\n    enquire_link PDU is send to \"ping\" the connetion. The default value is 30000 ms;\n    - `:enquire_link_resp_limit` is the maximum time for which ESME waits for enquire_link PDU response. If the\n    response is not received within this interval of time and no activity from the peer occurs, the session is then considered\n    dead and the ESME stops. The default value is 30000 ms;\n    - `:inactivity_limit` is the maximum time for which the peer is allowed not to send PDUs (which are not response PDUs).\n    If no such PDUs are received within this interval of time, ESME stops.\n    The default is :infinity ms;\n    - `:response_limit` is the maximum time to wait for a response for a previously sent PDU. If the response is\n    not received within this interval, `handle_resp_timeout` callback is triggered for the original pdu. If the response\n    is received later, it is discarded. The default value is 60000 ms.\n    - `:response_limit_resolution` is the maximum time after reaching `response_limit` for which a PDU without\n    a response is not collected. Setting to `0` will make PDUs without responses to be collected\n    immediately after `response_limit` ms. Setting to greater values improve performance since\n    such PDUs are collected in batches. The default value is 100 ms.\n    - `:session_init_limit` is the maximum time for the session to be unbound.\n    If no bind request succeed within this interval of time, the session stops.\n    The default value is 10000 ms;\n* `:socket_opts` is a keyword list of ranch socket options, see ranch's options for more information\nIf `:esme_opts` list of options is ommited, all options take their default values.\nThe whole `opts` argument may also be ommited in order to start ESME with the defaults.\nThe returned value is either `{:ok, pid}` or `{:error, reason}`.","ref":"SMPPEX.ESME.html#start_link/4"},{"type":"module","title":"SMPPEX.ESME.Sync","doc":"`SMPPEX.ESME.Sync` is an ESME implementation of `SMPPEX.Session`. It allows to send PDUs\nto SMSCs in a syncronous way, i.e. blocking till the response PDU comes.\n\nOne can use with `SMPPEX.ESME.Sync` all\nmethods provided by `SMPPEX.Session` like `SMPPEX.Session.send_pdu/2`, etc.","ref":"SMPPEX.ESME.Sync.html"},{"type":"function","title":"SMPPEX.ESME.Sync.pdus/2","doc":"A nonblocking version of `wait_for_pdus/2`.\n\nThe difference is that it always immediately returns a list of items(maybe empty)\nand never returns `:timeout` or `:stop`.","ref":"SMPPEX.ESME.Sync.html#pdus/2"},{"type":"function","title":"SMPPEX.ESME.Sync.request/3","doc":"Syncronously sends a PDU and wait for at most `timeout` ms for a response PDU.\n\nThe default timeout is 5000 ms.\n\nThe result value is one of:\n* `{:ok, resp}`, where `resp` is a successfully received response PDU;\n* `:timeout` if the response PDU was not received within `timeout`;\n* `:stop` if the ESME stopped while a response was awaited;\n* `{:error, reason}` if the request failed.","ref":"SMPPEX.ESME.Sync.html#request/3"},{"type":"function","title":"SMPPEX.ESME.Sync.start_link/3","doc":"Starts `SMPPEX.ESME.Sync`.\n\n`opts` is a keyword list of `SMPPEX.ESME` options which is directly passed to\nthe underlying `SMPPEX.ESME.start_link/4` call.","ref":"SMPPEX.ESME.Sync.html#start_link/3"},{"type":"function","title":"SMPPEX.ESME.Sync.stop/1","doc":"Stops ESME.","ref":"SMPPEX.ESME.Sync.html#stop/1"},{"type":"function","title":"SMPPEX.ESME.Sync.wait_for_pdus/2","doc":"Syncronously wait for incoming PDUs. If the ESME already have some received PDUs,\nthey are returned immediately.\n\nThe default timeout is 5000 ms.\n\nThe result value is `:timeout`, `:stop` or a list of the following items:\n* `{:pdu, pdu}`, where `pdu` is an incoming PDU;\n* `{:resp, resp_pdu, original_pdu}` where `resp_pdu` is an incoming reply for a\npreviously sent `original_pdu`;\n* `{:timeout, pdu}` for `pdu`'s which have not received a response within ESME timeout;\n* `{:error, pdu, reason}` for outcoming PDUs which were not successfully sent due to `reason`;\n* `{:ok, pdu}` for outcoming PDUs which were successfully sent.\n\n`:timeout` returned value indicates that the ESME didn't receive any PDUs within `timeout`.\n`:stop` value indicates that the ESME stopped while waiting for incoming PDUs.","ref":"SMPPEX.ESME.Sync.html#wait_for_pdus/2"},{"type":"type","title":"SMPPEX.ESME.Sync.awaited/0","doc":"","ref":"SMPPEX.ESME.Sync.html#t:awaited/0"},{"type":"module","title":"SMPPEX.MC","doc":"This is a module for launching a TCP listener (or any other listener supported by `ranch`, for example, `ssl`) which handles incoming connections with the passed `SMPPEX.Session` implementations.\n\nTo start an MC one generally should do the following.\n\n1. Implement an `SMPPEX.Session` behaviour.\n\n```elixir\n\ndefmodule MyMCSession do\n  use SMPPEX.Session\n\n  # ...Callback implementation\n\nend\n\n```\n\n2. Pass the child specification to a supervisor, using implemented behaviour as a session module:\n\n```elixir\nSupervisor.start_link(\n  [\n    {\n      SMPPEX.MC,\n      session: {MyESMESession, session_arg},\n      transport_opts: [port: 2775]\n    },\n    ...\n  ],\n  ...\n)\n```\n\nNote that each received connection is served with its own process which uses passed callback module (`MyESMESession`) for handling connection events. Each process has his own state initialized by `init` callback receiving `socket`, `transport` and a copy of arguments (`session_arg`).","ref":"SMPPEX.MC.html"},{"type":"function","title":"SMPPEX.MC.child_spec/1","doc":"Returns a supervisor child specification for starting listener for MC entity.\n\nStarting under a supervisor:\n\n```elixir\nSupervisor.start_link(\n  [\n    {SMPPEX.MC, session: {MyESMESession, session_arg}, ...},\n    ...\n  ],\n  ...\n)\n```\n\nOptions:\n\n* `:session` (required) a `{module, arg}` tuple, where `module` is the callback module\nwhich should implement `SMPPEX.Session` behaviour, while `arg` is the argument passed\nto the `init` callback each time a new connection is received.\n* `:transport` is :ranch transport used for TCP connections: either `ranch_tcp` (the default)\nor `ranch_ssl`;\n* `:transport_opts` is a map of :ranch transport options.\nThe major key is `socket_opts` which contains a list of important options such as `{:port, port}`.\nThe port is set to `0` by default, which means that the listener will accept\nconnections on a random free port. For backward compatibility one can pass a list of socket options\ninstead of `transport_opts` map (as in :ranch 1.x).\n* `:session_module` is a module to use as an alternative to `SMPPEX.Session`\nfor handling sessions (if needed). For example, `SMPPEX.TelemetrySession`.\n* `:acceptor_count` is the number of :ranch listener acceptors, 50 by default.\n* `:mc_opts` is a keyword list of MC options:\n    - `:session_init_limit` is the maximum time for which a session waits an incoming bind request.\n    If no bind request is received within this interval of time, the session stops.\n    The default value is 10000 ms;\n    - `:enquire_link_limit` is value for enquire_link SMPP timer, i.e. the interval of SMPP session\n    inactivity after which enquire_link PDU is send to \"ping\" the connetion.\n    The default value is 30000 ms;\n    - `:enquire_link_resp_limit` is the maximum time for which a session waits for enquire_link PDU response.\n    If the response is not received within this interval of time and no activity from the peer occurs,\n    the session is then considered dead and the session stops.\n    The default value is 30000 ms;\n    - `:inactivity_limit` is the maximum time for which a peer is allowed not to send PDUs\n    (which are not response PDUs). If no such PDUs are received within this interval of time,\n    the session stops. The default is :infinity ms;\n    - `:response_limit` is the maximum time to wait for a response for a previously sent PDU.\n    If the response is not received within this interval, `handle_resp_timeout` callback is triggered\n    for the original pdu. If the response is received later, it is discarded.\n    The default value is 60000 ms.\n    - `:response_limit_resolution` is the maximum time after reaching `response_limit` for which a PDU without\n    a response is not collected. Setting to `0` will make PDUs without responses to be collected\n    immediately after `response_limit` ms. Setting to greater values improve performance since\n    such PDUs are collected in batches. The default value is 100 ms.\n    - `:default_call_timeout` is an integer greater than zero which specifies how many milliseconds to wait\n    for a reply, or the atom :infinity to wait indefinitely.If no reply is received within the specified time,\n    the function call fails and the caller exits. The default value\n    is 5000 ms.\nIf `:mc_opts` list of options is ommited, all options take their default values.\nThe returned value is either `{:ok, ref}` or `{:error, reason}`. The `ref` can be later used\nto stop the whole MC listener and all sessions received by it.","ref":"SMPPEX.MC.html#child_spec/1"},{"type":"function","title":"SMPPEX.MC.start/2","doc":"Starts listener for MC entity.\n\nThe listener is started in the supervision tree of the `:ranch` application.\nTherefore, prefer `child_spec/1`, which allows you to start the MC in your own supervision tree.\n\nThe first argument must be a `{module, arg}` tuple, where `module` is the callback module which should implement `SMPPEX.Session` behaviour, while `arg` is the argument passed to the `init` callback each time a new connection is received.\nFor the list of other options see `child_spec/1`.","ref":"SMPPEX.MC.html#start/2"},{"type":"function","title":"SMPPEX.MC.stop/1","doc":"Stops MC listener and all its sessions.","ref":"SMPPEX.MC.html#stop/1"},{"type":"module","title":"SMPPEX.Pdu","doc":"Module for working with Pdu struct representing parsed SMPP PDU","ref":"SMPPEX.Pdu.html"},{"type":"function","title":"SMPPEX.Pdu.as_reply_to/2","doc":"Makes `pdu` be reply to the `reply_to_pdu`, i.e. assigns `reply_to_pdu`'s\n`sequence_number` to `pdu`.","ref":"SMPPEX.Pdu.html#as_reply_to/2"},{"type":"function","title":"Examples - SMPPEX.Pdu.as_reply_to/2","doc":"iex(1)> pdu1 = SMPPEX.Pdu.new({0x00000004, 0, 123})\n    iex(2)> pdu2 = SMPPEX.Pdu.new(0x80000004) |> SMPPEX.Pdu.as_reply_to(pdu1)\n    iex(3)> SMPPEX.Pdu.sequence_number(pdu2)\n    123","ref":"SMPPEX.Pdu.html#as_reply_to/2-examples"},{"type":"function","title":"SMPPEX.Pdu.bind?/1","doc":"Checks if Pdu is a bind request.","ref":"SMPPEX.Pdu.html#bind?/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.bind?/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4)\n    iex(2)> SMPPEX.Pdu.bind?(pdu)\n    false\n    iex(3)> pdu = SMPPEX.Pdu.new(1)\n    iex(4)> SMPPEX.Pdu.bind?(pdu)\n    true","ref":"SMPPEX.Pdu.html#bind?/1-examples"},{"type":"function","title":"SMPPEX.Pdu.bind_resp?/1","doc":"Checks if Pdu is a bind response.","ref":"SMPPEX.Pdu.html#bind_resp?/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.bind_resp?/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(0x80000004)\n    iex(2)> SMPPEX.Pdu.bind_resp?(pdu)\n    false\n    iex(3)> pdu = SMPPEX.Pdu.new(0x80000001)\n    iex(4)> SMPPEX.Pdu.bind_resp?(pdu)\n    true","ref":"SMPPEX.Pdu.html#bind_resp?/1-examples"},{"type":"function","title":"SMPPEX.Pdu.command_id/1","doc":"Returns Pdu's `command_id`.","ref":"SMPPEX.Pdu.html#command_id/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.command_id/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(1)\n    iex(2)> SMPPEX.Pdu.command_id(pdu)\n    1","ref":"SMPPEX.Pdu.html#command_id/1-examples"},{"type":"function","title":"SMPPEX.Pdu.command_name/1","doc":"Returns Pdu's symbolic command name as an atom or `:unknown` if Pdu's `command_id`\ndo not correspond to any real SMPP command.","ref":"SMPPEX.Pdu.html#command_name/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.command_name/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(1)\n    iex(2)> SMPPEX.Pdu.command_name(pdu)\n    :bind_receiver\n    iex(3)> pdu = SMPPEX.Pdu.new(1111111)\n    iex(4)> SMPPEX.Pdu.command_name(pdu)\n    :unknown","ref":"SMPPEX.Pdu.html#command_name/1-examples"},{"type":"function","title":"SMPPEX.Pdu.command_status/1","doc":"Returns Pdu's `command_status`.","ref":"SMPPEX.Pdu.html#command_status/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.command_status/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new({1, 4, 123})\n    iex(2)> SMPPEX.Pdu.command_status(pdu)\n    4","ref":"SMPPEX.Pdu.html#command_status/1-examples"},{"type":"function","title":"SMPPEX.Pdu.dest/1","doc":"Returns Pdu's `:destination_addr`, `:dest_addr_ton` and `:dest_addr_npi` fields\nin a tuple.","ref":"SMPPEX.Pdu.html#dest/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.dest/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{destination_addr: \"to\", dest_addr_ton: 1, dest_addr_npi: 2})\n    iex(2)> SMPPEX.Pdu.dest(pdu)\n    {\"to\", 1, 2}","ref":"SMPPEX.Pdu.html#dest/1-examples"},{"type":"function","title":"SMPPEX.Pdu.field/2","doc":"Get Pdu mandatory or optional(TLV) field by name or by integer id. If Pdu does not have the\nfield or field name is unknown, `nil` is returned.","ref":"SMPPEX.Pdu.html#field/2"},{"type":"function","title":"Examples - SMPPEX.Pdu.field/2","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{short_message: \"hi\"}, %{0x0424 => \"hello\"})\n    iex(2)> SMPPEX.Pdu.field(pdu, :message_payload)\n    \"hello\"\n    iex(3)> SMPPEX.Pdu.field(pdu, 0x0424)\n    \"hello\"\n    iex(4)> SMPPEX.Pdu.field(pdu, :short_message)\n    \"hi\"\n    iex(5)> SMPPEX.Pdu.field(pdu, :unknown_name)\n    nil","ref":"SMPPEX.Pdu.html#field/2-examples"},{"type":"function","title":"SMPPEX.Pdu.mandatory_field/2","doc":"Get Pdu mandatory field. If Pdu does not have the field, `nil` is returned.","ref":"SMPPEX.Pdu.html#mandatory_field/2"},{"type":"function","title":"Examples - SMPPEX.Pdu.mandatory_field/2","doc":"iex(1)> pdu = SMPPEX.Pdu.new({1, 4, 123}, %{system_id: \"system_id\"})\n    iex(2)> SMPPEX.Pdu.mandatory_field(pdu, :system_id)\n    \"system_id\"\n    iex(3)> SMPPEX.Pdu.mandatory_field(pdu, :short_message)\n    nil","ref":"SMPPEX.Pdu.html#mandatory_field/2-examples"},{"type":"function","title":"SMPPEX.Pdu.mandatory_fields/1","doc":"Get the whole set of mandatory fields as a map.","ref":"SMPPEX.Pdu.html#mandatory_fields/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.mandatory_fields/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{short_message: \"hi\"}, %{0x0424 => \"hello\"})\n    iex(2)> SMPPEX.Pdu.mandatory_fields(pdu)\n    %{short_message: \"hi\"}","ref":"SMPPEX.Pdu.html#mandatory_fields/1-examples"},{"type":"function","title":"SMPPEX.Pdu.new/3","doc":"Construct a new Pdu from header, mandatory fields and optional(TLV) fields.\n\nHeader may be either an integer, then it is treated as command id,\nor a tuple `{command_id, command_status, sequence_number}`\n\nEach Pdu is created with a unique ref field, by which one can later\ntrace Pdu's identity.","ref":"SMPPEX.Pdu.html#new/3"},{"type":"function","title":"Examples - SMPPEX.Pdu.new/3","doc":"iex(1)> SMPPEX.Pdu.new(1)\n    %SMPPEX.Pdu{command_id: 1, command_status: 0, mandatory: %{}, optional: %{},\n    ref: #Reference<0.0.3.215>, sequence_number: 0}\n    iex(2)> SMPPEX.Pdu.new({1, 0, 123}, %{system_id: \"sid\", password: \"pass\"}, %{})\n    %SMPPEX.Pdu{command_id: 1, command_status: 0,\n    mandatory: %{password: \"pass\", system_id: \"sid\"}, optional: %{},\n    ref: #Reference<0.0.3.219>, sequence_number: 123}","ref":"SMPPEX.Pdu.html#new/3-examples"},{"type":"function","title":"SMPPEX.Pdu.optional_field/2","doc":"Get Pdu optional(TLV) field by name or by integer id. If Pdu does not have the\nfield or field name is unknown, `nil` is returned.","ref":"SMPPEX.Pdu.html#optional_field/2"},{"type":"function","title":"Examples - SMPPEX.Pdu.optional_field/2","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{}, %{0x0424 => \"hello\"})\n    iex(2)> SMPPEX.Pdu.optional_field(pdu, :message_payload)\n    \"hello\"\n    iex(3)> SMPPEX.Pdu.optional_field(pdu, 0x0424)\n    \"hello\"\n    iex(4)> SMPPEX.Pdu.optional_field(pdu, :receipted_message_id)\n    nil\n    iex(5)> SMPPEX.Pdu.optional_field(pdu, :unknown_tlv_name)\n    nil","ref":"SMPPEX.Pdu.html#optional_field/2-examples"},{"type":"function","title":"SMPPEX.Pdu.optional_fields/1","doc":"Get the whole set of optional(TLV) fields as a map.","ref":"SMPPEX.Pdu.html#optional_fields/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.optional_fields/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{short_message: \"hi\"}, %{0x0424 => \"hello\"})\n    iex(2)> SMPPEX.Pdu.optional_fields(pdu)\n    %{0x0424 => \"hello\"}","ref":"SMPPEX.Pdu.html#optional_fields/1-examples"},{"type":"function","title":"SMPPEX.Pdu.ref/1","doc":"Returns Pdu's unique reference `ref`.","ref":"SMPPEX.Pdu.html#ref/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.ref/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new({1, 4, 123})\n    iex(2)> is_reference SMPPEX.Pdu.ref(pdu)\n    true","ref":"SMPPEX.Pdu.html#ref/1-examples"},{"type":"function","title":"SMPPEX.Pdu.resp?/1","doc":"Checks if Pdu is a response Pdu.","ref":"SMPPEX.Pdu.html#resp?/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.resp?/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4)\n    iex(2)> SMPPEX.Pdu.resp?(pdu)\n    false\n    iex(3)> pdu = SMPPEX.Pdu.new(0x80000004)\n    iex(4)> SMPPEX.Pdu.resp?(pdu)\n    true","ref":"SMPPEX.Pdu.html#resp?/1-examples"},{"type":"function","title":"SMPPEX.Pdu.same?/2","doc":"Checks if two Pdus are copies of the same Pdu.","ref":"SMPPEX.Pdu.html#same?/2"},{"type":"function","title":"Examples - SMPPEX.Pdu.same?/2","doc":"iex(1)> pdu1 = SMPPEX.Pdu.new(4)\n    iex(2)> pdu2 = SMPPEX.Pdu.new(4)\n    iex(3)> SMPPEX.Pdu.same?(pdu1, pdu2)\n    false\n    iex(4)> SMPPEX.Pdu.same?(pdu1, pdu1)\n    true","ref":"SMPPEX.Pdu.html#same?/2-examples"},{"type":"function","title":"SMPPEX.Pdu.sequence_number/1","doc":"Returns Pdu's `sequence_number`.","ref":"SMPPEX.Pdu.html#sequence_number/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.sequence_number/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new({1, 4, 123})\n    iex(2)> SMPPEX.Pdu.sequence_number(pdu)\n    123","ref":"SMPPEX.Pdu.html#sequence_number/1-examples"},{"type":"function","title":"SMPPEX.Pdu.set_mandatory_field/3","doc":"Sets Pdu mandatory field. New Pdu is returned.","ref":"SMPPEX.Pdu.html#set_mandatory_field/3"},{"type":"function","title":"Examples - SMPPEX.Pdu.set_mandatory_field/3","doc":"iex(1)> pdu = SMPPEX.Pdu.new({1, 4, 123}, %{system_id: \"system_id\"})\n    iex(2)> pdu1 = SMPPEX.Pdu.set_mandatory_field(pdu, :password, \"pass\")\n    iex(3)> SMPPEX.Pdu.mandatory_field(pdu1, :password)\n    \"pass\"","ref":"SMPPEX.Pdu.html#set_mandatory_field/3-examples"},{"type":"function","title":"SMPPEX.Pdu.set_optional_field/3","doc":"Sets Pdu optional field. New Pdu is returned.","ref":"SMPPEX.Pdu.html#set_optional_field/3"},{"type":"function","title":"Examples - SMPPEX.Pdu.set_optional_field/3","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4)\n    iex(2)> pdu1 = SMPPEX.Pdu.set_optional_field(pdu, :message_payload, \"hello\")\n    iex(3)> SMPPEX.Pdu.optional_field(pdu1, 0x0424)\n    \"hello\"","ref":"SMPPEX.Pdu.html#set_optional_field/3-examples"},{"type":"function","title":"SMPPEX.Pdu.source/1","doc":"Returns Pdu's `:source_addr`, `:source_addr_ton` and `:source_addr_npi` fields\nin a tuple.","ref":"SMPPEX.Pdu.html#source/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.source/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new(4, %{source_addr: \"from\", source_addr_ton: 1, source_addr_npi: 2})\n    iex(2)> SMPPEX.Pdu.source(pdu)\n    {\"from\", 1, 2}","ref":"SMPPEX.Pdu.html#source/1-examples"},{"type":"function","title":"SMPPEX.Pdu.success_resp?/1","doc":"Checks if Pdu is a successful response Pdu.","ref":"SMPPEX.Pdu.html#success_resp?/1"},{"type":"function","title":"Examples - SMPPEX.Pdu.success_resp?/1","doc":"iex(1)> pdu = SMPPEX.Pdu.new({0x80000004, 1, 0})\n    iex(2)> SMPPEX.Pdu.success_resp?(pdu)\n    false\n    iex(3)> pdu = SMPPEX.Pdu.new({0x80000004, 0, 0})\n    iex(4)> SMPPEX.Pdu.success_resp?(pdu)\n    true","ref":"SMPPEX.Pdu.html#success_resp?/1-examples"},{"type":"type","title":"SMPPEX.Pdu.addr/0","doc":"","ref":"SMPPEX.Pdu.html#t:addr/0"},{"type":"type","title":"SMPPEX.Pdu.header/0","doc":"","ref":"SMPPEX.Pdu.html#t:header/0"},{"type":"type","title":"SMPPEX.Pdu.t/0","doc":"","ref":"SMPPEX.Pdu.html#t:t/0"},{"type":"module","title":"SMPPEX.Pdu.Errors","doc":"Module for operating with SMPP errors.","ref":"SMPPEX.Pdu.Errors.html"},{"type":"function","title":"SMPPEX.Pdu.Errors.code_by_name/1","doc":"Converts atom representing SMPP error to its integer value.","ref":"SMPPEX.Pdu.Errors.html#code_by_name/1"},{"type":"function","title":"Example: - SMPPEX.Pdu.Errors.code_by_name/1","doc":"iex(1)> SMPPEX.Pdu.Errors.code_by_name(:ROK)\n    0","ref":"SMPPEX.Pdu.Errors.html#code_by_name/1-example"},{"type":"function","title":"SMPPEX.Pdu.Errors.description/1","doc":"Converts integer SMPP error code to text error description.","ref":"SMPPEX.Pdu.Errors.html#description/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Errors.description/1","doc":"iex(1)> SMPPEX.Pdu.Errors.description(0)\n    \"No Error\"\n    iex(2)> SMPPEX.Pdu.Errors.description(12345)\n    \"12345\"","ref":"SMPPEX.Pdu.Errors.html#description/1-example"},{"type":"function","title":"SMPPEX.Pdu.Errors.format/1","doc":"Converts integer SMPP error code to string error identifier.","ref":"SMPPEX.Pdu.Errors.html#format/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Errors.format/1","doc":"iex(1)> SMPPEX.Pdu.Errors.format(0)\n    \"ROK\"\n    iex(2)> SMPPEX.Pdu.Errors.format(12345)\n    \"12345\"","ref":"SMPPEX.Pdu.Errors.html#format/1-example"},{"type":"type","title":"SMPPEX.Pdu.Errors.error_code/0","doc":"","ref":"SMPPEX.Pdu.Errors.html#t:error_code/0"},{"type":"type","title":"SMPPEX.Pdu.Errors.error_name/0","doc":"","ref":"SMPPEX.Pdu.Errors.html#t:error_name/0"},{"type":"module","title":"SMPPEX.Pdu.Factory","doc":"Module for convenient generation of the most common PDUs.","ref":"SMPPEX.Pdu.Factory.html"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind/2","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind/2"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind/4","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind/4"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_receiver/3","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_receiver/3"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_receiver_resp/2","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_receiver_resp/2"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_resp/3","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_resp/3"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_transceiver/3","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_transceiver/3"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_transceiver_resp/2","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_transceiver_resp/2"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_transmitter/3","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_transmitter/3"},{"type":"function","title":"SMPPEX.Pdu.Factory.bind_transmitter_resp/2","doc":"","ref":"SMPPEX.Pdu.Factory.html#bind_transmitter_resp/2"},{"type":"function","title":"SMPPEX.Pdu.Factory.deliver_sm/3","doc":"","ref":"SMPPEX.Pdu.Factory.html#deliver_sm/3"},{"type":"function","title":"SMPPEX.Pdu.Factory.deliver_sm_resp/1","doc":"","ref":"SMPPEX.Pdu.Factory.html#deliver_sm_resp/1"},{"type":"function","title":"SMPPEX.Pdu.Factory.delivery_report/5","doc":"","ref":"SMPPEX.Pdu.Factory.html#delivery_report/5"},{"type":"function","title":"SMPPEX.Pdu.Factory.delivery_report_for_submit_sm/4","doc":"","ref":"SMPPEX.Pdu.Factory.html#delivery_report_for_submit_sm/4"},{"type":"function","title":"SMPPEX.Pdu.Factory.enquire_link/0","doc":"","ref":"SMPPEX.Pdu.Factory.html#enquire_link/0"},{"type":"function","title":"SMPPEX.Pdu.Factory.enquire_link_resp/1","doc":"","ref":"SMPPEX.Pdu.Factory.html#enquire_link_resp/1"},{"type":"function","title":"SMPPEX.Pdu.Factory.submit_sm/4","doc":"","ref":"SMPPEX.Pdu.Factory.html#submit_sm/4"},{"type":"function","title":"SMPPEX.Pdu.Factory.submit_sm_resp/2","doc":"","ref":"SMPPEX.Pdu.Factory.html#submit_sm_resp/2"},{"type":"function","title":"SMPPEX.Pdu.Factory.unbind/0","doc":"","ref":"SMPPEX.Pdu.Factory.html#unbind/0"},{"type":"function","title":"SMPPEX.Pdu.Factory.unbind_resp/1","doc":"","ref":"SMPPEX.Pdu.Factory.html#unbind_resp/1"},{"type":"type","title":"SMPPEX.Pdu.Factory.message/0","doc":"","ref":"SMPPEX.Pdu.Factory.html#t:message/0"},{"type":"type","title":"SMPPEX.Pdu.Factory.message_state/0","doc":"","ref":"SMPPEX.Pdu.Factory.html#t:message_state/0"},{"type":"module","title":"SMPPEX.Pdu.MessageState","doc":"Module for operating with `deliver_sm` message states.","ref":"SMPPEX.Pdu.MessageState.html"},{"type":"function","title":"SMPPEX.Pdu.MessageState.code_by_name/1","doc":"Converts atom representing message state to integer value.","ref":"SMPPEX.Pdu.MessageState.html#code_by_name/1"},{"type":"function","title":"Example - SMPPEX.Pdu.MessageState.code_by_name/1","doc":"iex(1)> SMPPEX.Pdu.MessageState.code_by_name(:DELIVERED)\n    2","ref":"SMPPEX.Pdu.MessageState.html#code_by_name/1-example"},{"type":"function","title":"SMPPEX.Pdu.MessageState.format/1","doc":"Converts integer message state value to string representation.","ref":"SMPPEX.Pdu.MessageState.html#format/1"},{"type":"function","title":"Example - SMPPEX.Pdu.MessageState.format/1","doc":"iex(1)> SMPPEX.Pdu.MessageState.format(2)\n    \"DELIVERED\"\n    iex(2)> SMPPEX.Pdu.MessageState.format(12345)\n    \"12345\"","ref":"SMPPEX.Pdu.MessageState.html#format/1-example"},{"type":"type","title":"SMPPEX.Pdu.MessageState.state/0","doc":"","ref":"SMPPEX.Pdu.MessageState.html#t:state/0"},{"type":"type","title":"SMPPEX.Pdu.MessageState.state_name/0","doc":"","ref":"SMPPEX.Pdu.MessageState.html#t:state_name/0"},{"type":"module","title":"SMPPEX.Pdu.Multipart","doc":"Module for operating with multipart information packed as UDH in message body.","ref":"SMPPEX.Pdu.Multipart.html"},{"type":"function","title":"SMPPEX.Pdu.Multipart.extract/1","doc":"Extracts multipart information from PDU or directly from binary message. This function is deprecated. Use `extract_from_pdu/1` or `extract_from_message/1` instead.\n\nReturns one of the following:\n* `{:ok, :single, message}` if the `message` does not contain any multipart information and represents a\nsingle message;\n* `{:ok, {ref_num, count, seq_num}, message}` if the original message contains multipart information in\nUDH fields. The outcoming `message` is cleared from UDH bytes.\n* `{:error, reason}`","ref":"SMPPEX.Pdu.Multipart.html#extract/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.extract/1","doc":"iex> data = <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.Multipart.extract(data)\n    {:ok, {3,2,1}, \"message\"}\n\n    iex> data = <<0x06, 0x08, 0x04, 0x00, 0x03, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.Multipart.extract(data)\n    {:ok, {3,2,1}, \"message\"}\n\n    iex> pdu = Pdu.new({1,0,1}, %{esm_class: 0b01000000, short_message: <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>})\n    iex> SMPPEX.Pdu.Multipart.extract(pdu)\n    {:ok, {3,2,1}, \"message\"}\n\n    iex> pdu = Pdu.new({1,0,1}, %{short_message: <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>})\n    iex> SMPPEX.Pdu.Multipart.extract(pdu)\n    {:error, \"Message is not multipart\"}","ref":"SMPPEX.Pdu.Multipart.html#extract/1-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.extract_from_ies/1","doc":"Extracts multipart information from already parsed list of UDH IEs.\n\nReturn one of the following:\n* `{:ok, :single}` if IEs do not contain any multipart message related ones;\n* `{:ok, {ref_num, count, seq_num}}` if there are multipart message related IEs (the first is taken);\n* `{:error, reason}` in case of errors.","ref":"SMPPEX.Pdu.Multipart.html#extract_from_ies/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.extract_from_ies/1","doc":"iex> ies = [{0, <<0x03, 0x02, 0x01>>}]\n    iex> SMPPEX.Pdu.Multipart.extract_from_ies(ies)\n    {:ok, {3, 2, 1}}\n\n    iex> ies = [{0, <<0x03, 0x02, 0x01>>}, {8, <<0x00, 0x04, 0x02, 0x01>>}]\n    iex> SMPPEX.Pdu.Multipart.extract_from_ies(ies)\n    {:ok, {3, 2, 1}}\n\n    iex> ies = [{8, <<0x00, 0x03, 0x02, 0x01>>}]\n    iex> SMPPEX.Pdu.Multipart.extract_from_ies(ies)\n    {:ok, {3, 2, 1}}\n\n    iex> ies = [{8, <<0x00, 0x03, 0x02>>}]\n    iex> SMPPEX.Pdu.Multipart.extract_from_ies(ies)\n    {:error, \"Invalid 16bit refrence number concatenated short messages info\"}\n\n    iex> ies = []\n    iex> SMPPEX.Pdu.Multipart.extract_from_ies(ies)\n    {:ok, :single}","ref":"SMPPEX.Pdu.Multipart.html#extract_from_ies/1-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.extract_from_message/1","doc":"Extracts multipart information from binary message.\n\nReturns one of the following:\n* `{:ok, :single, message}` if the `message` does not contain any multipart information and represents a\nsingle message. The outcoming `message` is cleared from UDH bytes.\n* `{:ok, {ref_num, count, seq_num}, message}` if the original message contains multipart information in\nUDH fields. The outcoming `message` is cleared from UDH bytes.\n* `{:error, reason}`","ref":"SMPPEX.Pdu.Multipart.html#extract_from_message/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.extract_from_message/1","doc":"iex> data = <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.Multipart.extract_from_message(data)\n    {:ok, {3,2,1}, \"message\"}\n\n    iex> data = <<0x06, 0x08, 0x04, 0x00, 0x03, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.Multipart.extract_from_message(data)\n    {:ok, {3,2,1}, \"message\"}","ref":"SMPPEX.Pdu.Multipart.html#extract_from_message/1-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.extract_from_pdu/1","doc":"Extracts multipart information from PDU.\n\nReturns one of the following:\n* `{:ok, :single, message}` if the `message` does not contain any multipart information and represents a\nsingle message. The outcoming `message` is cleared from UDH bytes.\n* `{:ok, {ref_num, count, seq_num}, message}` if the original message contains multipart information in\nUDH fields. The outcoming `message` is cleared from UDH bytes.\n* `{:error, reason}`","ref":"SMPPEX.Pdu.Multipart.html#extract_from_pdu/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.extract_from_pdu/1","doc":"iex> pdu = Pdu.new({1,0,1}, %{esm_class: 0b01000000, short_message: <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>})\n    iex> SMPPEX.Pdu.Multipart.extract_from_pdu(pdu)\n    {:ok, {3,2,1}, \"message\"}\n\n    iex> pdu = Pdu.new({1,0,1}, %{short_message: <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>})\n    iex> SMPPEX.Pdu.Multipart.extract_from_pdu(pdu)\n    {:error, \"Message is not multipart\"}","ref":"SMPPEX.Pdu.Multipart.html#extract_from_pdu/1-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.multipart_ie/1","doc":"Generates IE encoding multipart information.","ref":"SMPPEX.Pdu.Multipart.html#multipart_ie/1"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.multipart_ie/1","doc":"iex> SMPPEX.Pdu.Multipart.multipart_ie({3,2,1})\n    {:ok, {0, <<0x03, 0x02, 0x01>>}}\n\n    iex> SMPPEX.Pdu.Multipart.multipart_ie({256,2,1})\n    {:ok, {8, <<0x01, 0x00, 0x02, 0x01>>}}\n\n    iex> SMPPEX.Pdu.Multipart.multipart_ie({1, 1, 256})\n    {:error, \"Invalid sequence number in multipart info\"}","ref":"SMPPEX.Pdu.Multipart.html#multipart_ie/1-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.prepend_message_with_part_info/2","doc":"Prepends message with multipart info encoded as UDH.","ref":"SMPPEX.Pdu.Multipart.html#prepend_message_with_part_info/2"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.prepend_message_with_part_info/2","doc":"iex> SMPPEX.Pdu.Multipart.prepend_message_with_part_info({3,2,1}, \"message\")\n    {:ok, <<0x05, 0x00, 0x03, 0x03, 0x02, 0x01, \"message\">>}\n\n    iex> SMPPEX.Pdu.Multipart.prepend_message_with_part_info({256,2,1}, \"message\")\n    {:ok, <<0x06, 0x08, 0x04, 0x01, 0x00, 0x02, 0x01, \"message\">>}","ref":"SMPPEX.Pdu.Multipart.html#prepend_message_with_part_info/2-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.split_message/3","doc":"Splits message into parts prepending each part with multipart information UDH\nso that the resulting size of each part does not exceed `max_len` bytes.\n\nThe result is one of the following:\n* `{:ok, :unsplit}` if the message already fits into `max_len` bytes;\n* `{:ok, :split, parts}` if the message was succesfully split into `parts`;\n* `{:error, reason}` in case of errors.","ref":"SMPPEX.Pdu.Multipart.html#split_message/3"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.split_message/3","doc":"iex> SMPPEX.Pdu.Multipart.split_message(123, \"abc\", 3)\n    {:ok, :unsplit}\n\n    iex> SMPPEX.Pdu.Multipart.split_message(123, \"abcdefg\", 6)\n    {:error, \"Invalid limits for splitting message\"}\n\n    iex> SMPPEX.Pdu.Multipart.split_message(123, \"abcdefghi\", 8)\n    {:ok, :split, [\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x01, \"ab\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x02, \"cd\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x03, \"ef\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x04, \"gh\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x05, \"i\">>\n    ]}","ref":"SMPPEX.Pdu.Multipart.html#split_message/3-example"},{"type":"function","title":"SMPPEX.Pdu.Multipart.split_message/4","doc":"Splits message into parts not exceeding `max_split` bytes and prepending each part with multipart information UDH.\nThe message is not split if its size does not exceed `max_len` bytes.\n\nThe results format is the same as in `split_message/3`.","ref":"SMPPEX.Pdu.Multipart.html#split_message/4"},{"type":"function","title":"Example - SMPPEX.Pdu.Multipart.split_message/4","doc":"iex> SMPPEX.Pdu.Multipart.split_message(123, \"abcdefghi\", 0, 2)\n    {:ok, :split, [\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x01, \"ab\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x02, \"cd\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x03, \"ef\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x04, \"gh\">>,\n      <<0x05, 0x00, 0x03, 0x7b, 0x05, 0x05, \"i\">>\n    ]}","ref":"SMPPEX.Pdu.Multipart.html#split_message/4-example"},{"type":"type","title":"SMPPEX.Pdu.Multipart.actual_part_info/0","doc":"","ref":"SMPPEX.Pdu.Multipart.html#t:actual_part_info/0"},{"type":"type","title":"SMPPEX.Pdu.Multipart.extract_result/0","doc":"","ref":"SMPPEX.Pdu.Multipart.html#t:extract_result/0"},{"type":"type","title":"SMPPEX.Pdu.Multipart.part_info/0","doc":"","ref":"SMPPEX.Pdu.Multipart.html#t:part_info/0"},{"type":"type","title":"SMPPEX.Pdu.Multipart.split_result/0","doc":"","ref":"SMPPEX.Pdu.Multipart.html#t:split_result/0"},{"type":"module","title":"SMPPEX.Pdu.NetworkErrorCode","doc":"Module for operating with `deliver_sm` network_error_code parameter.","ref":"SMPPEX.Pdu.NetworkErrorCode.html"},{"type":"function","title":"SMPPEX.Pdu.NetworkErrorCode.decode/1","doc":"Converts octet_string from network_error_code tag to type_code and error_value","ref":"SMPPEX.Pdu.NetworkErrorCode.html#decode/1"},{"type":"function","title":"Example - SMPPEX.Pdu.NetworkErrorCode.decode/1","doc":"iex(1)> SMPPEX.Pdu.NetworkErrorCode.decode(<<08,00,01>>)\n    {8, 1}","ref":"SMPPEX.Pdu.NetworkErrorCode.html#decode/1-example"},{"type":"function","title":"SMPPEX.Pdu.NetworkErrorCode.encode/2","doc":"Converts network_error_code type and error to octet string","ref":"SMPPEX.Pdu.NetworkErrorCode.html#encode/2"},{"type":"function","title":"Example - SMPPEX.Pdu.NetworkErrorCode.encode/2","doc":"iex(1)> SMPPEX.Pdu.NetworkErrorCode.encode(8,1)\n    <<08,00,01>>","ref":"SMPPEX.Pdu.NetworkErrorCode.html#encode/2-example"},{"type":"type","title":"SMPPEX.Pdu.NetworkErrorCode.error_code/0","doc":"","ref":"SMPPEX.Pdu.NetworkErrorCode.html#t:error_code/0"},{"type":"type","title":"SMPPEX.Pdu.NetworkErrorCode.network_error_code/0","doc":"","ref":"SMPPEX.Pdu.NetworkErrorCode.html#t:network_error_code/0"},{"type":"type","title":"SMPPEX.Pdu.NetworkErrorCode.type_code/0","doc":"","ref":"SMPPEX.Pdu.NetworkErrorCode.html#t:type_code/0"},{"type":"module","title":"SMPPEX.Pdu.PP","doc":"Module for colored pretty printing Pdu structs.","ref":"SMPPEX.Pdu.PP.html"},{"type":"function","title":"SMPPEX.Pdu.PP.format/3","doc":"Forms an iolist containing colored Pdu dump.\n\n`indent` is the string prepended to each line of the dump (\"  \" by default).\n`pad` is the string prepended to nested lines of the dump together with `indent`.\nThe default is \"\".","ref":"SMPPEX.Pdu.PP.html#format/3"},{"type":"function","title":"Example - SMPPEX.Pdu.PP.format/3","doc":"iex> pdu = SMPPEX.Pdu.Factory.submit_sm({\"from\", 1, 1}, {\"to\", 1, 1}, \"hello\")\n\nThen `pdu |> SMPPEX.Pdu.PP.format |> IO.puts()` will print:\n\n```\npdu: submit_sm\n  command_id: 4\n  command_status: 0 (ok)\n  sequence_number: 0\nmandatory fields:\n  dest_addr_npi: 1\n  dest_addr_ton: 1\n  destination_addr: \"to\"\n  registered_delivery: 0\n  short_message: \"hello\"\n  source_addr: \"from\"\n  source_addr_npi: 1\n  source_addr_ton: 1\noptional fields: []\n```","ref":"SMPPEX.Pdu.PP.html#format/3-example"},{"type":"module","title":"SMPPEX.Pdu.TONNPIDefaults","doc":"Module for automatic TON/NPI detection","ref":"SMPPEX.Pdu.TONNPIDefaults.html"},{"type":"function","title":"SMPPEX.Pdu.TONNPIDefaults.ton_npi/1","doc":"","ref":"SMPPEX.Pdu.TONNPIDefaults.html#ton_npi/1"},{"type":"module","title":"SMPPEX.Pdu.UDH","doc":"Module for parsing encoded IEs from UDHs.","ref":"SMPPEX.Pdu.UDH.html"},{"type":"function","title":"SMPPEX.Pdu.UDH.add/2","doc":"Encodes IEs and prepends message with the encoded value.\n\nThe result is one of the following:\n* `{:ok, message}` where message is the original message prefixed with UDH;\n* `{:error, reason}` in case of errors.","ref":"SMPPEX.Pdu.UDH.html#add/2"},{"type":"function","title":"Example - SMPPEX.Pdu.UDH.add/2","doc":"iex> ies = [{0x05, <<0x06, 0x2d, 0x00, 0x00>>}, {0x00, <<0x01, 0x02, 0x01>>}]\n    iex> SMPPEX.Pdu.UDH.add(ies, \"message\")\n    {:ok, <<0x0B, 0x05, 0x04, 0x06, 0x2d, 0x00, 0x00, 0x00, 0x03, 0x01, 0x02, 0x01, \"message\">>}\n\n    iex> ies = [{0, 123}]\n    iex> SMPPEX.Pdu.UDH.add(ies, \"message\")\n    {:error, \"Invalid UDH IE data\"}\n\n    iex> ies = [{345, \"ie\"}]\n    iex> SMPPEX.Pdu.UDH.add(ies, \"message\")\n    {:error, \"Invalid UDH IE id\"}\n\n    iex> ies = [{-1, \"ie\"}]\n    iex> SMPPEX.Pdu.UDH.add(ies, \"message\")\n    {:error, \"Invalid UDH IE id\"}\n\n    iex> ies = [{0, <<1 :: integer-size(2040)>>}]\n    iex> SMPPEX.Pdu.UDH.add(ies, \"message\")\n    {:error, \"UDH is too long\"}","ref":"SMPPEX.Pdu.UDH.html#add/2-example"},{"type":"function","title":"SMPPEX.Pdu.UDH.extract/1","doc":"Extracts list of IEs from UDH.\n\nReturns one of the following\n* `{:ok, ies, message}` where `ies` is a list of IE tuples `{id, value}` and `message`\nis the original message without UDH;\n* `{:error, reason}` in case of errors.","ref":"SMPPEX.Pdu.UDH.html#extract/1"},{"type":"function","title":"Example - SMPPEX.Pdu.UDH.extract/1","doc":"iex> data = <<5, 0, 3, 197, 3, 3, \"message\">>\n    iex> SMPPEX.Pdu.UDH.extract(data)\n    {:ok, [{0, <<197, 3, 3>>}], \"message\"}\n\n    iex> data = <<0x0B, 0x05, 0x04, 0x06, 0x2d, 0x00, 0x00, 0x00, 0x03, 0x01, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.UDH.extract(data)\n    {:ok, [{0x05, <<0x06, 0x2d, 0x00, 0x00>>}, {0x00, <<0x01, 0x02, 0x01>>}], \"message\"}\n\n    iex> data = <<0x10, \"short\">>\n    iex> SMPPEX.Pdu.UDH.extract(data)\n    {:error, \"Invalid UDH length\"}\n\n    iex> data = <<0x06, 0x00, 0x03, 0x01, 0x02, 0x01, \"message\">>\n    iex> SMPPEX.Pdu.UDH.extract(data)\n    {:error, \"Invalid UDH data\"}\n\n    iex> data = <<5, 0, 4, 197, 3, 3, \"message\">>\n    iex> SMPPEX.Pdu.UDH.extract(data)\n    {:error, \"Invalid UDH IE length\"}","ref":"SMPPEX.Pdu.UDH.html#extract/1-example"},{"type":"function","title":"SMPPEX.Pdu.UDH.has_udh?/1","doc":"Checks if message in PDU has UDH (by inspecting `esm_class` field).","ref":"SMPPEX.Pdu.UDH.html#has_udh?/1"},{"type":"function","title":"Example - SMPPEX.Pdu.UDH.has_udh?/1","doc":"iex> pdu = SMPPEX.Pdu.new({1,0,1}, %{esm_class: 0}, %{})\n    iex> SMPPEX.Pdu.UDH.has_udh?(pdu)\n    false\n\n    iex> pdu = SMPPEX.Pdu.new({1,0,1}, %{esm_class: 0b01000000}, %{})\n    iex> SMPPEX.Pdu.UDH.has_udh?(pdu)\n    true","ref":"SMPPEX.Pdu.UDH.html#has_udh?/1-example"},{"type":"type","title":"SMPPEX.Pdu.UDH.ie/0","doc":"","ref":"SMPPEX.Pdu.UDH.html#t:ie/0"},{"type":"module","title":"SMPPEX.Pdu.ValidityPeriod","doc":"Module for converting validity period to Unix timestamp.\n\nModule works both with the absolute format of validity period and with the relative one.\nIn case of relative validity period, this module implements a naive representation of the month and year date shifting\nfor the sake of simplicity.","ref":"SMPPEX.Pdu.ValidityPeriod.html"},{"type":"function","title":"SMPPEX.Pdu.ValidityPeriod.to_timestamp/2","doc":"Converts `t:validity_period/0` to Unix timestamp according to the SMPP specification.\n\nIn case of the relative format, this function uses a naive implementation of the month and date shifting.\nTo be clear, it takes one month as 30 days and one year as 12 months.\n\nOne who uses the function should implement the way how this time shift might be limited\naccording to the SMPP specification:\n\n* A MC operator may choose to impose a limit on\nrelative time offsets, thus either rejecting a message that exceeds such a limit or reducing the\noffset to the maximum relative time allowed.\n\nReturns `{:ok, timestamp}` if conversion was successful.\n\nReturns `{:error, :invalid_validity_period}` if `validity_period` is not consistent with the SMPP specification.\n\nIn case of internal errors, however, this function raises an exception.","ref":"SMPPEX.Pdu.ValidityPeriod.html#to_timestamp/2"},{"type":"function","title":"Example (relative format) - SMPPEX.Pdu.ValidityPeriod.to_timestamp/2","doc":"iex> timestamp_origin = ~N[2017-01-01 00:00:00] |>\n    ...> DateTime.from_naive!(\"Etc/UTC\") |>\n    ...> DateTime.to_unix()\n    iex> timestamp = SMPPEX.Pdu.ValidityPeriod.to_timestamp!(\"000000000005000R\", timestamp_origin)\n    iex> DateTime.from_unix!(timestamp) |> to_string()\n    \"2017-01-01 00:00:05Z\"","ref":"SMPPEX.Pdu.ValidityPeriod.html#to_timestamp/2-example-relative-format"},{"type":"function","title":"Example (absolute format) - SMPPEX.Pdu.ValidityPeriod.to_timestamp/2","doc":"iex> {:ok, timestamp} = SMPPEX.Pdu.ValidityPeriod.to_timestamp(\"170610233429004+\")\n    iex> DateTime.from_unix!(timestamp) |> to_string()\n    \"2017-06-10 22:34:29Z\"","ref":"SMPPEX.Pdu.ValidityPeriod.html#to_timestamp/2-example-absolute-format"},{"type":"function","title":"SMPPEX.Pdu.ValidityPeriod.to_timestamp!/2","doc":"Converts `t:validity_period/0` to Unix timestamp according to the SMPP specification.\nThe same as `to_timestamp/2` but raises an exception in case of error.","ref":"SMPPEX.Pdu.ValidityPeriod.html#to_timestamp!/2"},{"type":"type","title":"SMPPEX.Pdu.ValidityPeriod.timestamp/0","doc":"","ref":"SMPPEX.Pdu.ValidityPeriod.html#t:timestamp/0"},{"type":"type","title":"SMPPEX.Pdu.ValidityPeriod.timestamp_origin/0","doc":"","ref":"SMPPEX.Pdu.ValidityPeriod.html#t:timestamp_origin/0"},{"type":"type","title":"SMPPEX.Pdu.ValidityPeriod.validity_period/0","doc":"","ref":"SMPPEX.Pdu.ValidityPeriod.html#t:validity_period/0"},{"type":"behaviour","title":"SMPPEX.Session","doc":"Module for implementing custom SMPP Session entities.\n\nTo implement an Session entitiy, one should implement several callbacks (`SMPPEX.Session` behaviour).\nThe most proper way to do it is to `use` `SMPPEX.Session`:\n\n```\ndefmodule MySession do\n  use SMPPEX.Session\n\n  # ...Callback implementation\n\nend\n```\n\nIn this case all callbacks have reasonable defaults.","ref":"SMPPEX.Session.html"},{"type":"function","title":"SMPPEX.Session.call/3","doc":"Makes a syncronous call to the session.\n\nThe call is handled by `handle_call/3` `SMPPEX.Session` callback.","ref":"SMPPEX.Session.html#call/3"},{"type":"function","title":"SMPPEX.Session.cast/2","doc":"Makes an asyncronous call to Session.\n\nThe call is handled by `handle_cast/2` `SMPPEX.Session` callback.","ref":"SMPPEX.Session.html#cast/2"},{"type":"callback","title":"SMPPEX.Session.code_change/3","doc":"Invoked to change the state of the session when a different version of a module is loaded (hot code swapping) and the state’s term structure should be changed. The method has the same semantics as the original `GenServer.code_change/3` callback.","ref":"SMPPEX.Session.html#c:code_change/3"},{"type":"callback","title":"SMPPEX.Session.handle_call/3","doc":"Invoked to handle an arbitrary syncronous `request` sent to the session with `Session.call/3` method.\n\n`from` argument can be used to send a response asyncronously via `Session.reply/2`.\n\nThe returned values indicate the following:\n* `{:reply, reply, state}` — reply with `reply` and use `state` as the new state;\n* `{:reply, reply, pdus, state}`  — reply with `reply`, use `state` as the new state and additionally send `pdus` to the peer;\n* `{:noreply, state}` — do not reply and use `state` as the new state. The reply can be send later via `Session.reply`;\n* `{:noreply, pdus, state}` — do not reply, use `state` as the new state and additionally send `pdus` to the peer. The reply can be send later via `Session.reply`;\n* `{:stop, reason, reply, state}` — reply with `reply`, use `state` as the new state and exit with `reason`;\n* `{:stop, reason, state}` — do not reply, use `state` as the new state and exit with `reason`.","ref":"SMPPEX.Session.html#c:handle_call/3"},{"type":"callback","title":"SMPPEX.Session.handle_cast/2","doc":"Invoked to handle an arbitrary asyncronous `request` sent to the session with `Session.cast/2` method.\n\nThe returned values indicate the following:\n* `{:noreply, state}` — use `state` as the new state;\n* `{:noreply, pdus, state}` — use `state` as the new state and additionally send `pdus` to the peer.;\n* `{:stop, reason, state}` — use `state` as the new state and exit with `reason`.","ref":"SMPPEX.Session.html#c:handle_cast/2"},{"type":"callback","title":"SMPPEX.Session.handle_info/2","doc":"Invoked to handle a generic message `request` sent to the session process.\n\nThe returned values indicate the following:\n* `{:noreply, state}` — use `state` as the new state;\n* `{:noreply, pdus, state}` — use `state` as the new state and additionally send `pdus` to the peer.;\n* `{:stop, reason, state}` — use `state` as the new state and exit with `reason`.","ref":"SMPPEX.Session.html#c:handle_info/2"},{"type":"callback","title":"SMPPEX.Session.handle_pdu/2","doc":"Invoked when the session receives an incoming PDU (which is not a response PDU).\n\nThe callback return values indicate the following:\n* `{:ok, state}` — use `state` as the new session state;\n* `{:ok, pdus, state}` — use `state` as the new session state and additionally send `pdus` to the connection;\n* `{:stop, reason, state}` — stop with reason `reason` and use `state` as the new session state.","ref":"SMPPEX.Session.html#c:handle_pdu/2"},{"type":"callback","title":"SMPPEX.Session.handle_resp/3","doc":"Invoked when the session receives a response to a previously sent PDU.\n\n`pdu` argument contains the received response PDU, `original_pdu` contains\nthe previously sent pdu for which the handled response is received.\n\nThe callback return values indicate the following:\n* `{:ok, state}` — use `state` as the new session state;\n* `{:ok, pdus, state}` — use `state` as the new session state and additionally send `pdus` to the connection;\n* `{:stop, reason, state}` — stop with reason `reason` and use `state` as the new session state.","ref":"SMPPEX.Session.html#c:handle_resp/3"},{"type":"callback","title":"SMPPEX.Session.handle_resp_timeout/2","doc":"Invoked when the session does not receive a response to a previously sent PDU\nwithin the specified timeout.\n\n`pdu` argument contains the PDU for which no response was received. If the response\nwill be received later it will be dropped (with an `info` log message).\n\nThe callback return values indicate the following:\n* `{:ok, state}` — use `state` as the new session state;\n* `{:ok, pdus, state}` — use `state` as the new session state and additionally send `pdus` to the connection;\n* `{:stop, reason, state}` — stop with reason `reason` and use `state` as the new session state.","ref":"SMPPEX.Session.html#c:handle_resp_timeout/2"},{"type":"callback","title":"SMPPEX.Session.handle_send_pdu_result/3","doc":"Invoked when the SMPP session successfully sent PDU to transport or failed to do this.\n\n`pdu` argument contains the PDU for which send status is reported. `send_pdu_result` can be\neither `:ok` or `{:error, reason}`.\n\nThe returned value is used as the new state.","ref":"SMPPEX.Session.html#c:handle_send_pdu_result/3"},{"type":"callback","title":"SMPPEX.Session.handle_socket_closed/1","doc":"Invoked when the connection is closed by the peer.\n\nThe returned value should be `{reason, state}`. The session stops then with `reason`.","ref":"SMPPEX.Session.html#c:handle_socket_closed/1"},{"type":"callback","title":"SMPPEX.Session.handle_socket_error/2","doc":"Invoked when the connection's socket reported a error.\n\nThe returned value should be `{reason, state}`. The session stops then with `reason`.","ref":"SMPPEX.Session.html#c:handle_socket_error/2"},{"type":"callback","title":"SMPPEX.Session.handle_timeout/2","doc":"Invoked when the session is about to be stopped due to a timeout.\n\nThe return value will be used as the exit reason of the session process. The default implementation returns `{:timers, reason}`.","ref":"SMPPEX.Session.html#c:handle_timeout/2"},{"type":"callback","title":"SMPPEX.Session.handle_unparsed_pdu/3","doc":"Invoked when the session receives an incoming PDU which couldn't be correctly parsed.\n\nThe callback return values indicate the following:\n* `{:ok, state}` — use `state` as the new session state;\n* `{:ok, pdus, state}` — use `state` as the new session state and additionally send `pdus` to the connection;\n* `{:stop, reason, state}` — stop with reason `reason` and use `state` as the new session state.","ref":"SMPPEX.Session.html#c:handle_unparsed_pdu/3"},{"type":"callback","title":"SMPPEX.Session.init/3","doc":"Invoked when a session is started after a connection successfully established.\n\n`args` argument is taken directly from `ESME.start_link` or `MC.start` call.\nThe return value should be either `{:ok, state}`, then the session will successfully start and the returned state will be later passed to the other callbacks, or `{:stop, reason}`, then the session will stop with the returned reason.","ref":"SMPPEX.Session.html#c:init/3"},{"type":"function","title":"SMPPEX.Session.reply/2","doc":"Replies to a client calling `Session.call` method.\n\nThis function can be used to explicitly send a reply to a client that called `call/3`.\n\n`from` must be the `from` argument (the second argument) accepted by `handle_call/3` callbacks.\n\n The return value is always `:ok`.","ref":"SMPPEX.Session.html#reply/2"},{"type":"function","title":"SMPPEX.Session.send_pdu/3","doc":"Sends a PDU from the session to the peer.","ref":"SMPPEX.Session.html#send_pdu/3"},{"type":"function","title":"SMPPEX.Session.stop/2","doc":"Stops the session syncronously.","ref":"SMPPEX.Session.html#stop/2"},{"type":"callback","title":"SMPPEX.Session.terminate/3","doc":"Invoked when the session process is about to exit.\n\n`lost_pdus` contain a list of nonresp `pdus` sent by the session to the peer and which have not yet received a response.\n\nThe returned value is either `:stop` or `{:stop, last_pdus, state}`, where `last_pdus` is a list of PDUs which will be sent to the peer before socket close, and `state` is the new state. For example, an ESME can send an unbind PDU or an MC can send negative resps for pending `submit_sm`s if needed.\n\nThis callback is called from the underlying `GenServer` `terminate` callbacks, so it has all the corresponding caveats, for example, sometimes it may not be called, see [`GenServer.terminate/2` docs](https://hexdocs.pm/elixir/GenServer.html#c:terminate/2).","ref":"SMPPEX.Session.html#c:terminate/3"},{"type":"type","title":"SMPPEX.Session.from/0","doc":"","ref":"SMPPEX.Session.html#t:from/0"},{"type":"type","title":"SMPPEX.Session.reason/0","doc":"","ref":"SMPPEX.Session.html#t:reason/0"},{"type":"type","title":"SMPPEX.Session.reply/0","doc":"","ref":"SMPPEX.Session.html#t:reply/0"},{"type":"type","title":"SMPPEX.Session.request/0","doc":"","ref":"SMPPEX.Session.html#t:request/0"},{"type":"type","title":"SMPPEX.Session.send_pdu_result/0","doc":"","ref":"SMPPEX.Session.html#t:send_pdu_result/0"},{"type":"type","title":"SMPPEX.Session.session/0","doc":"","ref":"SMPPEX.Session.html#t:session/0"},{"type":"type","title":"SMPPEX.Session.state/0","doc":"","ref":"SMPPEX.Session.html#t:state/0"},{"type":"extras","title":"SMPPEX","doc":"# SMPPEX","ref":"examples.html"},{"type":"extras","title":"General description - SMPPEX","doc":"SMPPEX is a framework for building SMPP servers and clients (which are often\nreferred to as MC and ESME entities respectevely).\n\nThe major features exposed by the library are:\n\n* `SMPPEX.ESME` module and behaviour for implementing ESME entities;\n* `SMPPEX.MC` module and behaviour for implementing MC entities;\n* `SMPPEX.ESME.Sync` module representing simple ready to use SMPP client.\n\nAlso one of the core features of the library is simplicity: both code simplicity and simplicity of use.\n\n* The library does not have much TCP handling or session management functionality,\nit is based on great [`ranch`](https://github.com/ninenines/ranch) library.\n* SMPP session is symmetric(used both in ESME and MC) and is implemented as\n`ranch_protocol` behaviour.\n* The library includes an easy and ready to use SMPP client (`SMPP.ESME.Sync`) which\nhas capabilities of synchronous SMS sending and do not require implementing ESME behavior.\nThere is also an SMPP testing tool [`smppsend`](https://github.com/funbox/smppsend)\nbased on this client.","ref":"examples.html#general-description"},{"type":"extras","title":"SMPPEX.ESME.Sync - SMPPEX","doc":"`SMPPEX.ESME.Sync` is the most straightforward way to interact with an SMSC. Example:\n\n```elixir\n\n{:ok, esme} = SMPPEX.ESME.Sync.start_link(host, port)\n\nbind = SMPPEX.Pdu.Factory.bind_transmitter(\"system_id\", \"password\")\n{:ok, _bind_resp} = SMPPEX.ESME.Sync.request(esme, bind)\n\n# We are bound, let's send a message\n\nsubmit_sm = SMPPEX.Pdu.Factory.submit_sm({\"from\", 1, 1}, {\"to\", 1, 1}, \"hello!\")\n{:ok, submit_sm_resp} = SMPPEX.ESME.Sync.request(esme, submit_sm)\n\n# Message is sent, let's get the obtained id:\n\nmessage_id = SMPPEX.Pdu.field(submit_sm_resp, :message_id)\n\n# Now let's wait for a delivery report:\n\ndelivery_report? = fn(pdu) ->\nSMPPEX.Pdu.command_name(pdu) == :deliver_sm and\n  SMPPEX.Pdu.field(pdu, :receipted_message_id) == message_id\nend\n\ndelivery_reports = case SMPPEX.ESME.Sync.wait_for_pdus(esme, 60000) do\n:stop ->\n  Logger.info(\"Ooops, ESME stopped\")\n  []\n:timeout ->\n  Logger.info(\"No DLR in 60 seconds\")\n  []\nreceived_items ->\n  # Let's filter out DLRs for the previously submitted message\n  for {:pdu, pdu}  <- received_items, delivery_report?.(pdu), do: pdu\nend\n```","ref":"examples.html#smppex-esme-sync"},{"type":"extras","title":"SMPPEX.ESME - SMPPEX","doc":"`SMPPEX.ESME` can be used when more complicated client logic is needed, for example\ncustom immediate reactions to all incoming PDUs, rps/window control, etc.\n\n`SMPPEX.Session` provides \"empty\" defaults for all required callbacks, so minimal ESME\ncould be very simple:\n\n```elixir\ndefmodule DummyESME do\n  use SMPPEX.Session\n\n  def start_link(host, port) do\n    SMPPEX.ESME.start_link(host, port, {__MODULE__, []})\n  end\nend\n```\n\nIt is still completely functional:\n\n```elixir\n{:ok, esme} = DummyESME.start_link(host, port)\nSMPPEX.Session.send_pdu(esme, SMPPEX.Pdu.Factory.bind_transmitter(\"system_id\", \"password\"))\n```\n\nHere's a more complicated example of ESME, which does the following:\n\n* Receives port number and three arguments:\n\n    - `waiting_pid` — a pid of the process which will be informed when ESME stops;\n    - `count` — count of PDUs to send;\n    - `window` — window size, the maximum number of sent PDU's without resps.\n\n* Connects to the specified port on localhost and issues a bind command.\n* Starts to send predefined PDUs after bind at maximum possible rate but regarding window size.\n* Stops after all PDUs are sent and notifies the waiting process.\n\n```elixir\ndefmodule SMPPBenchmarks.ESME do\n\n  use SMPPEX.Session\n  require Logger\n\n  @from {\"from\", 1, 1}\n  @to {\"to\", 1, 1}\n  @message \"hello\"\n\n  @system_id \"system_id\"\n  @password \"password\"\n\n  def start_link(port, waiting_pid, count, window) do\n    SMPPEX.ESME.start_link(\"127.0.0.1\", port, {__MODULE__, [waiting_pid, count, window]})\n  end\n\n  def init(_, _, [waiting_pid, count, window]) do\n    Kernel.send(self(), :bind)\n    {:ok, %{waiting_pid: waiting_pid, count_to_send: count, count_waiting_resp: 0, window: window}}\n  end\n\n  def handle_resp(pdu, _original_pdu, st) do\n    case SMPPEX.Pdu.command_name(pdu) do\n    :submit_sm_resp ->\n      new_st = %{ st | count_waiting_resp: st.count_waiting_resp - 1 }\n      send_pdus(new_st)\n    :bind_transmitter_resp ->\n      send_pdus(st)\n    _ ->\n      {:ok, st}\n    end\n  end\n\n  def handle_resp_timeout(pdu, st) do\n    Logger.error(\"PDU timeout: #{inspect pdu}, terminating\")\n    {:stop, :resp_timeout, st}\n\n  end\n\n  def terminate(reason, _, st) do\n    Logger.info(\"ESME stopped with reason #{inspect reason}\")\n    Kernel.send(st.waiting_pid, {self(), :done})\n    :stop\n  end\n\n  def handle_info(:bind, st) do\n    {:noreply, [SMPPEX.Pdu.Factory.bind_transmitter(@system_id, @password)], st}\n  end\n\n  defp send_pdus(st) do\n    cond do\n      st.count_to_send > 0 ->\n        count_to_send = min(st.window - st.count_waiting_resp, st.count_to_send)\n        new_st = %{ st | count_waiting_resp: st.window, count_to_send: st.count_to_send - count_to_send }\n        {:ok, make_pdus(count_to_send), new_st}\n      st.count_waiting_resp > 0 ->\n        {:ok, st}\n      true ->\n        Logger.info(\"All PDUs sent, all resps received, terminating\")\n        {:stop, :normal, st}\n    end\n  end\n\n  defp make_pdus(0), do: []\n  defp make_pdus(n) do\n    for _ <- 1..n, do: SMPPEX.Pdu.Factory.submit_sm(@from, @to, @message)\n  end\n\nend\n```\n\nNot all callbacks are used yet in this example, for the full list see `SMPPEX.Session` documentation.","ref":"examples.html#smppex-esme"},{"type":"extras","title":"SMPPEX.MC - SMPPEX","doc":"`SMPPEX.MC` is used for _receiving_ and handling SMPP connections.\n\nHere is an example of a very simple MC, which does the following:\n\n* Starts and listens to connections on the specified port.\n* Responds with OK status to all incoming binds.\n* Responds with incremental message ids to all incoming `submit_sm` packets (regardless of the bind state).\n\n```elixir\n\ndefmodule MC do\n\n  use SMPPEX.Session\n\n  alias SMPPEX.Pdu\n  alias SMPPEX.Pdu.Factory, as: PduFactory\n\n  def child_spec(port) do\n    Supervisor.child_spec(\n      {\n        SMPPEX.MC,\n        session: {__MODULE__, []},\n        transport_opts: [port: port]\n      },\n      []\n    )\n  end\n\n  def init(_socket, _transport, []) do\n    {:ok, 0}\n  end\n\n  def handle_pdu(pdu, last_id) do\n    case Pdu.command_name(pdu) do\n      :submit_sm ->\n        {:ok, [PduFactory.submit_sm_resp(0, to_string(last_id)) |> Pdu.as_reply_to(pdu)], last_id + 1}\n      :bind_transmitter ->\n        {:ok, [PduFactory.bind_transmitter_resp(0) |> Pdu.as_reply_to(pdu)], last_id}\n      _ ->\n        {:ok, last_id}\n    end\n  end\n\nend\n```\n\nThis server can be started by providing `{MC, port}` as a child of some supervisor.","ref":"examples.html#smppex-mc"},{"type":"extras","title":"Related projects","doc":"# Related projects\n\nBesides the core `SMPPEX` project there are some auxilary projects.","ref":"projects.html"},{"type":"extras","title":"smppsend - Related projects","doc":"[smppsend](https://github.com/funbox/smppsend) is a simple CLI utility\nfor testing SMSC connections. It allows to bind to SMSCs, send submit_sm\nPDUs and wait for delivery reports.","ref":"projects.html#smppsend"},{"type":"extras","title":"smppex_web - Related projects","doc":"[smppex_web](https://github.com/savonarola/smppex_web) is a simple demonstrational MC (SMPP server)\nwith web interface.\n\nA public installation is available online at:\n* Web: [smppex.rubybox.ru](https://smppex.rubybox.ru).\n* SMPP: `smppex.rubybox.ru:2775`\n\nIt is useful for manual preliminary testing of SMPP clients.\n\nOne can connect to the demonstrational MC with the mentioned `smppssend` tool:\n\n```\n./smppsend --host smppex.rubybox.ru --port 2775 --system-id systemid --password pass --bind-mode trx --submit-sm --source-addr from123 --source-addr-npi 1 --source-addr-ton 5 --destination-addr 99999999999 --dest-addr-npi 1 --dest-addr-ton 1 --short-message  HelloHelloHelloHelloHelloHelloHelloHelloHello --data-coding 8 --split-max-bytes 30 --udh-ref 123 --ucs2 --registered-delivery 1 --wait\n\n19:57:33.677 [info]  Connecting to smppex.rubybox.ru:2775\n\n19:57:33.749 [info]  Connected\n\n19:57:33.753 [info]  Binding\npdu: bind_transceiver\n  command_id: 9\n  command_status: 0 (ok)\n  sequence_number: 0\nmandatory fields:\n  addr_npi: nil\n  addr_ton: nil\n  address_range: nil\n  interface_version: nil\n  password: \"pass\"\n  system_id: \"systemid\"\n  system_type: nil\noptional fields: []\n\n...\n```","ref":"projects.html#smppex_web"},{"type":"extras","title":"smpp_benchmarks - Related projects","doc":"[smpp_benchmarks](https://github.com/savonarola/smpp_benchmarks) is a bunch of\nbenchmarks to run against SMPPEX.","ref":"projects.html#smpp_benchmarks"},{"type":"extras","title":"smppex_telemetry - Related projects","doc":"[smppex_telemetry](https://github.com/savonarola/smppex_telemetry) is an advanced\nimplementation of `SMPPEX.Session` which emits [telemetry](https://github.com/beam-telemetry/telemetry) events.","ref":"projects.html#smppex_telemetry"},{"type":"extras","title":"smppex_graceful - Related projects","doc":"Sometimes it is important to shutdown one or many SMPP sessions \"gracefully\",\ni.e. to send `unbind` PDU and wait for a response before disconnect.\n\n[smppex_graceful](https://github.com/savonarola/smppex_graceful) contains an example\nof how one can organize ESME sessions to achieve this.","ref":"projects.html#smppex_graceful"},{"type":"extras","title":"Changelog","doc":"# Changelog\nAll notable changes to this project will be documented in this file.\n\nThe format is based on [Keep a Changelog](http://keepachangelog.com/en/1.0.0/)\nand this project adheres to [Semantic Versioning](http://semver.org/spec/v2.0.0.html).\n\n## [Unreleased]","ref":"changelog.html"},{"type":"extras","title":"Changed - Changelog","doc":"- Rework time related code of SMPP sessions.\n- Drop `oserl` support.\n\n## [3.2.1] - 2023-06-03\n\n- Drop support of old Elixir & OTP versions (OTP 24+ and Elixir 1.12+ are required).\n- Remove some compatibility code.\n- Update dependencies to latest minor versions.\n- Update CI matrix.\n- Improve SSL tests.\n- Update maintenance status.\n- Apply updated formatting.\n\n## [3.0.6] - 2022-01-22","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Embedded mode for MC, i.e. a way to start it as a part of custom supervision tree ([sasa1977](https://github.com/sasa1977)).\n- Optional `SMPP.Session` callback (`handle_timeout`) for customizing SMPP session\nexit reason in case of timeouts ([sasa1977](https://github.com/sasa1977)).\n\n## [3.0.5] - 2021-12-22","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Elixir 13 to CI matrix.\n- OTP 24 to CI matrix.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed some typing issues ([Fyodor Parmanchukov](https://github.com/rezerbit)).\n\n## [3.0.4] - 2021-04-17","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Documentation of auxilary projects.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Organization of documents.\n\n## [3.0.3] - 2021-04-16","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `session_module` option for MC and ESME modules to provide alternative implementations\nof `SMPPEX.TransportSession` behavior.\n\n## [3.0.2] - 2021-04-06","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Removed `poison` from dev dependencies.\n\n## [3.0.1] - 2020-12-13","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Removed `poison` from production dependencies.\n\n## [3.0.0] - 2020-12-13","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed callback flow while accepting connections in MC. `init` callbacks are now run\nafter socket transport handshake (as it always should have been).\n- Updated Ranch to 2.0.\n- Dropped OTP < 21 support for Ranch 2.0 compatibility.\n- Fixed `source_subaddress` TLV spec ([Menkir](https://github.com/Menkir)).\n\n## [2.4.0] - 2020-11-05","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed low level socket configuration in ESME ([IceDragon200](https://github.com/IceDragon200)).\n- Updated dev/test dependency versions.\n- Dropped Elixir < 1.7 support.\n\n## [2.3.3] - 2020-09-09","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed submit_multi handling ([Menkir](https://github.com/Menkir)).\n- Moved repo to [funbox](https://github.com/funbox) organization.\n- Set the `esm_class` field for a delivery report pdu created by factory.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Timeout argument to `Session.send_pdu/2` function.\n\n## [2.3.2] - 2019-11-19","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Option to set arbitrary initial sequence number for SMPP sessions.\n\n## [2.3.1] - 2019-09-19","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Elixir 1.9 builds in CI.\n- Updated `excoveralls` dependency.\n- Added `SMPPEX.Pdu.ValidityPeriod` module for dealing with `validity_period` PDU field.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Removed Elixir 1.2, 1.3 support and the corresponding builds from CI.\n\n## [2.3.0] - 2019-04-08","ref":"changelog.html#removed"},{"type":"extras","title":"Added - Changelog","doc":"- Elixir 1.8 builds in CI.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- OTP 18 support.","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed `data_sm` packet syntax.\n\n## [2.2.9] - 2018-11-22","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Unfixed `ranch` from `< 1.6.0`, since its internal changes do not affect our code.\n\n## [2.2.8] - 2018-11-11","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed automatic `enquire_link` sequence id generation.\n- Fixed `ranch` to `< 1.6.0` due to its incompatible changes.\n\n## [2.2.7] - 2018-06-08","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Parsing mudule for `network_error_code` field.\n- `network_error_code` field support in `oserl` converter.\n\n## [2.2.6] - 2018-06-05","ref":"changelog.html#added"},{"type":"extras","title":"Added - Changelog","doc":"- Handling of generic `GenServer` `call` and `cast` messages.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed handling of PDUs with negative `send_pdu_result` status: they do not appear\nin `handle_resp_timeout` callack anymore.\n\n## [2.2.5] - 2018-05-23","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- `submit_sm` factory methods with automatic TON/NPI detection.","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- Dropped Elixir 1.1.1 and OTP 17 support.\n- Made `SMPPEX.ESME.Sync` be safe for making requestd from multiple processes.\n\n## [2.2.4] - 2018-02-12","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Updated build matrix for Travis CI. Removed assets for obsolete version builds.\n- Updated build matrix for Travis CI. Added Elixir 1.6.\n- Added explicit extract functions in SMPPEX.Pdu.Multipart.\n- Fixed Ranch transport handling in SMPPEX.TransportSession, this fixes SSL transport support.\n\n## [2.2.3] - 2017-09-21","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- SMPPEX.ESME.Sync: ignore successful send_pdu_result for syncronously sent PDUs.\n- SMPPEX.ESME.Sync: exit with normal on socket close.\n\n## [2.2.2] - 2017-09-21","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Fixed handling socket close/error for SMPPEX.ESME.Sync\n\n## [2.2.1] - 2017-09-11","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Loosened `ranch` version requirements.\n\n## [2.2.0] - 2017-09-10","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added strict response type for `terminate` callback. Also, `terminate` callback is now allowed to return some last pdus for sending.\n\n## [2.1.0] - 2017-09-10","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Automatic handling of `enquire_link` and `enquire_link_resp` PDUs.\n\n## [2.0.1] - 2017-09-02","ref":"changelog.html#added"},{"type":"extras","title":"Changed - Changelog","doc":"- `PduStorage` implementation to be OTP 17 compatible.\n\n## [2.0.0] - 2017-08-30\nThis release contains significant architectural changes and API incompatibilities.","ref":"changelog.html#changed"},{"type":"extras","title":"Changed - Changelog","doc":"- Added single `SMPPEX.Session` behavior instead of seperate `SMPPEX.ESME` and `SMPPEX.MC` behaviours.\n- `SMPPEX.Session` callbacks are more `GenServer` compliant.\n- Most of `SMPPEX.Session` callbacks are allowed to return stop indicating tuple.\n- Most of `SMPPEX.Session` callbacks are allowed to return a list of PDUs to send.\n- All methods interacting with `SMPPEX.Session` are synchronous.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- Elixir 1.5 builds in CI.","ref":"changelog.html#added"},{"type":"extras","title":"Removed - Changelog","doc":"- Usage of a separate process for ESME/MC connections.\n- Usage of `ranch` supervisor of launching ESME connections.\n- Special methods for sending PDU replies to ESME/MC in favor of `Pdu.as_reply_to/2` method.\n\n## [1.0.1] - 2017-08-30","ref":"changelog.html#removed"},{"type":"extras","title":"Changed - Changelog","doc":"- `ex_doc` updated to 1.x.\n\n\n## [1.0.0] - 2017-08-30\nThis is the first stable release introduced from numerous 0.x.x versions. Although this library has been already used in production for quite a while, it's only purpose is to designate the divergence from 2.x.x branch.","ref":"changelog.html#changed"},{"type":"extras","title":"Added - Changelog","doc":"- ESME functionality.\n- MC functionality.\n- Synchronous ESME client.","ref":"changelog.html#added"},{"type":"extras","title":"LICENSE","doc":"MIT License\n\nCopyright 2017-2023 Ilya Averyanov\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the \"Software\"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED \"AS IS\", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.","ref":"license.html"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,50]}}